package com.bundle.bundle_rave_pay;

import android.Manifest;
import android.annotation.TargetApi;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Build;
import android.os.Bundle;
import android.os.Parcelable;
import android.util.Log;
import android.view.View;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.FragmentTransaction;

import com.bundle.bundle_rave_pay.account.AccountFragment;
import com.bundle.bundle_rave_pay.banktransfer.BankTransferFragment;
import com.bundle.bundle_rave_pay.card.CardFragment;
import com.bundle.bundle_rave_pay.di.components.AppComponent;
import com.bundle.bundle_rave_pay.di.modules.AndroidModule;
import com.bundle.bundle_rave_pay.di.modules.NetworkModule;
import com.bundle.bundle_rave_pay.di.components.DaggerAppComponent;

import static com.bundle.bundle_rave_pay.RaveConstants.ENCRYPTION_KEY;
import static com.bundle.bundle_rave_pay.RaveConstants.LIVE_URL;
import static com.bundle.bundle_rave_pay.RaveConstants.PAYMENT_TYPE_ACCOUNT;
import static com.bundle.bundle_rave_pay.RaveConstants.PAYMENT_TYPE_BANK_TRANSFER;
import static com.bundle.bundle_rave_pay.RaveConstants.PERMISSIONS_REQUEST_READ_PHONE_STATE;
import static com.bundle.bundle_rave_pay.RaveConstants.PUBLIC_KEY;
import static com.bundle.bundle_rave_pay.RaveConstants.RAVE_PARAMS;
import static com.bundle.bundle_rave_pay.RaveConstants.STAGING_URL;

public class RavePayActivity extends AppCompatActivity {

    public static String BASE_URL;
    RavePayInitializer ravePayInitializer;

    public static int RESULT_SUCCESS = 111;
    public static int RESULT_ERROR = 222;
    public static int RESULT_CANCELLED = 333;
    int theme;
    View permissionsRequiredLayout;

    AppComponent appComponent;

    Double amount = 0.0;
    String cvv = "";
    String cardExpiry = "";
    String cardNumber = "";
    boolean saveCard = true;
    String country = "";
    String currencyCode = "";
    String email = "";
    String fName = "";
    String lName = "";
    String narration = "";
    String txRef = "";
    boolean isLive = false;
    String publicKey = PUBLIC_KEY;
    String encryptionKey = ENCRYPTION_KEY;

    boolean isCard = false;
    boolean isAccount = false;
    boolean isTransfer = false;

    BundleCard bundleCard;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(com.bundle.bundle_rave_pay.R.layout.activity_rave_pay);

        BundleData bundleData;

        bundleData = getIntent().getParcelableExtra(RAVE_PARAMS);


        startCardPayment(bundleData);

        bundleCard = new BundleCard(amount, cvv, cardExpiry, cardNumber, saveCard);

        buildGraph();
        theme = ravePayInitializer.getTheme();

        if (theme != 0) {
            try {
                setTheme(theme);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            checkForRequiredPermissions();
        } else {
            setupMainContent();
        }


    }

    private void startCardPayment(BundleData cardBundle) {
        this.amount = cardBundle.getAmount();
        this.cvv = cardBundle.getCvv();
        this.cardExpiry = cardBundle.getCardExpiry();
        this.cardNumber = cardBundle.getCardNumber();
        this.saveCard = cardBundle.isSaveCard();
        this.country = cardBundle.getCountryCode();
        this.currencyCode = cardBundle.getCurrencyCode();
        this.email = cardBundle.getEmail();
        this.fName = cardBundle.getFirstName();
        this.lName = cardBundle.getLastName();
        this.txRef = cardBundle.getTransactionReference();
        this.isLive = cardBundle.isLive();
        this.publicKey = cardBundle.getPublicKey();
        this.encryptionKey = cardBundle.getEncryptionKey();

        this.isCard = cardBundle.isCard();
        this.isAccount = cardBundle.isAccount();
        this.isTransfer = cardBundle.isBankTransfer();

        ravePayInitializer = setupManager();
    }

    private RavePayInitializer setupManager() {
        RavePayManager ravePayManager = new RavePayManager(this).setAmount(amount)
                .setPublicKey(publicKey)
                .setEncryptionKey(encryptionKey)
                .setCountry(country)
                .setCurrency(currencyCode)
                .setEmail(email)
                .setfName(fName)
                .setlName(lName)
                .setNarration(narration)
                .setTxRef(txRef)
                .acceptAccountPayments(isAccount)
                .acceptCardPayments(isCard)
                .acceptBankTransferPayments(isTransfer)
                .onStagingEnv(!isLive)
                .shouldDisplayFee(false);

        return ravePayManager.createRavePayInitializer();
    }

    private void setupMainContent() {
        int paymentMethod = ravePayInitializer.getOrderedPaymentTypesList().get(0);

        launchFragment(paymentMethod, bundleCard);
    }

    private void launchFragment(int paymentType, Parcelable data) {
        FragmentTransaction transaction = getSupportFragmentManager().beginTransaction();

        switch (paymentType) {
            case PAYMENT_TYPE_ACCOUNT:
                transaction.replace(com.bundle.bundle_rave_pay.R.id.payment_fragment_container, new AccountFragment());
                break;
            case PAYMENT_TYPE_BANK_TRANSFER:
                transaction.replace(com.bundle.bundle_rave_pay.R.id.payment_fragment_container, new BankTransferFragment());
                break;
            case RaveConstants.PAYMENT_TYPE_CARD:
                Bundle bundle = new Bundle();

                bundle.putParcelable("card", bundleCard);
                CardFragment cardFragment = new CardFragment();
                cardFragment.setArguments(bundle);

                transaction.replace(com.bundle.bundle_rave_pay.R.id.payment_fragment_container, cardFragment);
                break;
            default:
                Log.d("Adding Payment Fragment", "Payment type does not exist in payment types list");
                return;
        }

        try {
            transaction.commit();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        setResult(resultCode, data);
    }

    private void buildGraph() {

        if (ravePayInitializer.isStaging()) {
            BASE_URL = STAGING_URL;
        } else {
            BASE_URL = LIVE_URL;
        }

        appComponent = DaggerAppComponent.builder()
                .androidModule(new AndroidModule(this))
                .networkModule(new NetworkModule(BASE_URL))
                .build();
    }


    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);

        if (requestCode == PERMISSIONS_REQUEST_READ_PHONE_STATE) {
            if (grantResults.length > 0
                    && grantResults[0] == PackageManager.PERMISSION_GRANTED) {

                permissionsRequiredLayout.setVisibility(View.GONE);
                setupMainContent();

            }
        }
    }

    @TargetApi(Build.VERSION_CODES.M)
    public void checkForRequiredPermissions() {

        if (checkSelfPermission(Manifest.permission.READ_PHONE_STATE)
                != PackageManager.PERMISSION_GRANTED) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                requestPermissions(new String[]{Manifest.permission.READ_PHONE_STATE},
                        PERMISSIONS_REQUEST_READ_PHONE_STATE);
            }
        } else {
            setupMainContent();
        }
    }

    @Override
    public void onBackPressed() {
        setResult(RavePayActivity.RESULT_CANCELLED, new Intent());
        super.onBackPressed();
    }

    public AppComponent getAppComponent() {
        return appComponent;
    }

    public RavePayInitializer getRavePayInitializer() {
        return ravePayInitializer;
    }

}